class Component:
    def __init__(self, **kwds):
        self.__dict__.update(kwds)
        self.relations = set()

    def addRelation(self, relation):
        self.relations.add(relation)

    def __repr__(self):
        return self.name

class Arm(Component):
    pass

class Frame(Component):
    pass

class Lens(Component):
    pass

class Bridge(Component):
    pass

class Empty(Component):
    def __init__(self):
        Component.__init__(self)
        
    def addRelation(self, relation):
        raise Exception("Empty component cannot be in a relation.")

class Relation:
    pass

class Attached(Relation):
    def __init__(self, comp1, comp2, create = True):
        self.comp1 = comp1
        self.comp2 = comp2
        if create:
            comp1.addRelation(self)
            comp2.addRelation(self)

    def __hash__(self):
        return hash(self.comp1) + hash(self.comp2)

    def __eq__(self, other):
        if isinstance(other, Attached):
            return {self.comp1, self.comp2} == {other.comp1, other.comp2}
        return NotImplemented
