from component import *

class Fact:
    def __init__(self, conf):
        self.conf = conf
    def activate(self, partial):
        pass
    def check(self, full):
        pass
    def __repr__(self):
        pass

class Exists(Fact):
    def __init__(self, conf, comp):
        Fact.__init__(self, conf)
        self.comp = comp

    def activate(self, partial):
        return self.comp not in partial

    def check(self, full):
        return not isinstance(self.comp, Empty)

    def __repr__(self):
        return "[{} exists]".format(self.comp)

class IsAttached(Fact):
    def __init__(self, conf, c1, c2):
        Fact.__init__(self, conf)
        self.c1 = c1
        self.c2 = c2

    def activate(self, partial):
        return any(c not in partial for c in {self.c1, self.c2})

    def check(self, components):
        if any(isinstance(c, Empty) for c in {self.c1, self.c2}):
            return True
        return Attached(self.c1, self.c2, False) in self.c1.relations

    def __repr__(self):
        return "[{} is attached to {}]".format(self.c1, self.c2)

class LengthEqual(Fact):
    def __init__(self, conf, c1, c2):
        Fact.__init__(self, conf)
        self.components = {c1, c2}

    def activate(self, partial):
        return any(c not in partial for c in self.components)

    def check(self, components):
        if any(isinstance(c, Empty) or not hasattr(c, 'length')
               for c in self.components):
            return True
        c1, c2 = self.components
        return c1.length == c2.length

    def __repr__(self):
        c1, c2 = self.components
        return "[{} and {} have equal length]".format(c1, c2)

class ColorEqual(Fact):
    def __init__(self, conf, c1, c2):
        Fact.__init__(self, conf)
        self.components = {c1, c2}

    def activate(self, partial):
        return any(c not in partial for c in self.components)

    def check(self, components):
        if any(isinstance(c, Empty) or not hasattr(c, 'color')
               for c in self.components):
            return True
        c1, c2 = self.components
        return c1.color == c2.color

    def __repr__(self):
        c1, c2 = self.components
        return "[{} and {} have the same color]".format(c1, c2)
