from component import *
from fact import *

def main(filename):
    with open(filename) as f:
        s = f.read(-1)
        exec(s, globals())

    sunglasses = {left_arm, left_frame, left_lens, bridge, right_frame,
                  right_lens, right_arm}

    left_arm.name = "Left Arm"
    right_arm.name = "Right Arm"
    left_frame.name = "Left Frame"
    right_frame.name = "Right Frame"
    left_lens.name = "Left Lens"
    right_lens.name = "Right Lens"
    bridge.name = "Bridge"


    knowledge = {Exists(.9, left_arm),
                 Exists(.9, right_arm),
                 Exists(.9, left_frame),
                 Exists(.9, right_frame),
                 Exists(.9, left_lens),
                 Exists(.9, right_lens),
                 Exists(.9, bridge),
                 IsAttached(.9, left_frame, left_arm),
                 IsAttached(.9, left_frame, left_lens),
                 IsAttached(.9, left_frame, bridge),
                 IsAttached(.9, right_frame, right_arm),
                 IsAttached(.9, right_frame, right_lens),
                 IsAttached(.9, right_frame, bridge),
                 LengthEqual(.7, right_arm, left_arm),
                 ColorEqual(.7, right_lens, left_lens)}

    active = {exp for exp in knowledge
              if exp.activate(partial)}

    active_violations = {exp for exp in active
                         if not exp.check(sunglasses)}

    passive_violations = {exp for exp in (knowledge - active)
                          if not exp.check(sunglasses)}

    total_surprise = (sum(v.conf for v in active_violations)
                      + sum(.7 * v.conf for v in passive_violations))

    normalized_surprise = total_surprise / len(knowledge)
                     

    print("surprise report for", filename)
    print("active expectation failures:")
    for v in active_violations:
        print("  ", v)
    print("passive expectation failures:")
    for v in passive_violations:
        print("  ", v)
    print("total surprise:", total_surprise)
    print("normalized total surprise:", normalized_surprise)
    print()

if __name__ == '__main__':
    main("missing_right_arm_active.sgl")
    main("missing_right_arm_passive.sgl")
    
    main("different_colored_lenses_active.sgl")
    main("different_colored_lenses_passive.sgl")

    main("short_left_arm_active.sgl")
    main("short_left_arm_passive.sgl")
    
    main("control_active.sgl")
    main("control_passive.sgl")
    
